package de.hft.stuttgart.citydoctor2.ziploader;

import de.hft.stuttgart.citydoctor2.check.Checker;
import de.hft.stuttgart.citydoctor2.datastructure.CityDoctorModel;
import de.hft.stuttgart.citydoctor2.parser.CityGmlParseException;
import de.hft.stuttgart.citydoctor2.parser.CityGmlParser;
import de.hft.stuttgart.citydoctor2.parser.InvalidGmlFileException;
import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

public class CityGmlZipEntry {

    private static final Logger logger = LogManager.getLogger(CityGmlZipEntry.class);

    private final String fileName;
    private final CityDoctorModel model;
    private CityGmlArchive parentArchive;
    private boolean validated = false;
    private ZipEntryErrorType errorType = null;


    public static CityGmlZipEntry of(ZipEntry entry, ZipFile archive, ParserConfiguration config){
        if ((double) entry.getSize() /1024/1024 > 20) {
            return new ErroneousEntry(entry, ZipEntryErrorType.EXCESSIVE_FILESIZE);
        } else {
            try {
                ZipInputStream zis = new ZipInputStream(archive.getInputStream(entry));
                CityGmlParser.gagLogger(true);
                CityDoctorModel model = CityGmlParser.parseCityGmlStream(zis, config);
                return new CityGmlZipEntry(entry.getName(), model);
            } catch (CityGmlParseException | InvalidGmlFileException e) {
                logger.error(e);
                return new ErroneousEntry(entry, ZipEntryErrorType.INVALID_CITY_GML_FILE);
            } catch (IOException e){
                logger.error(e);
                return new ErroneousEntry(entry, ZipEntryErrorType.IO_ERROR);
            }
        }
    }

    private CityGmlZipEntry(String filename, CityDoctorModel model){
        this.fileName = filename;
        this.model = model;
    }

    protected CityGmlZipEntry(ZipEntry entry) {
        this.fileName = entry.getName();
        this.model = null;
    }

    public void setArchive(CityGmlArchive archive){
        parentArchive = archive;
    }

    public CityGmlArchive getArchive(){
        return parentArchive;
    }

    public void validateModel(){
        Checker checker = new Checker(this.getModel());
        checker.runChecks();
        validated = true;
    }

    public boolean isValidated(){
        return validated;
    }

    public String getFileName() {
        return fileName;
    }

    public CityDoctorModel getModel() {
        return model;
    }

}
